#running the file will convert files results/genPhrase.pkl, results/genSentence.pkl and results/genWords.pkl into spikes (print them
#to the screen and save them as .sp file).  It will also print the sentence generated and generate a spike plot.

#you can pass this a pkl file name, e.g.
# python printGenResults.py results/genSentence.pkl spikeSentenceStates 0, which will convert results/genSentence.pkl to spike data, save the spikes to
#spikeSentenceStates.sp and will not generate a spikeplot (i.e. the last parameter is 0)
#
import sys
import cPickle as pickle
from matplotlib import pyplot as plt
from pyNN.utility.plotting import Figure, Panel
import numpy as np
import os

def printSentence(Words,WordNeuronsFired):
    #print the sentence reported by the neurons
    NumNeuronsFired=len(WordNeuronsFired)
    if(NumNeuronsFired>0):
        Sentence=[]

        for eachNeuron in range(0,NumNeuronsFired-1):
            Sentence+=[Words[WordNeuronsFired[eachNeuron]]]

        WholeSentence=" ".join(Sentence)+Words[WordNeuronsFired[NumNeuronsFired-1]]

    else:
        WholeSentence=""
    print "\nSentence From Network:"
    print WholeSentence

def RetrieveWords(wordFile):
    #retrieve the words from OutputWords.txt
    if (os.path.isfile(wordFile)):
        OutputWords=[]
        fileHandle = open(wordFile,'r')
        for inpLine in fileHandle:
            word=inpLine.split()
            OutputWords+=word
        return OutputWords

def SortNeurons(WordNeuronsFired,WordSpikeTimes):
        ##sort neurons into ascending spike time order
        sort_index = np.argsort(WordSpikeTimes)
        SortedNeuronsFired=[]
        for neuron in range(0,len(WordSpikeTimes)):
                SortedNeuronsFired+=[WordNeuronsFired[sort_index[neuron]]]

        return SortedNeuronsFired

def SpikeTrainGraph(fileName,saveas):
    fileHandle = open(fileName)
    neoObj = pickle.load(fileHandle)
    segments = neoObj.segments
    segment = segments[0]
    spikeTrains = segment.spiketrains
    
    Figure(
    Panel(spikeTrains, xlabel="Time (ms)", xticks=True, yticks=True),
    title=saveas
    ).save("results/"+saveas) #save in the results directory
    plt.show()

def printPklSpikes(fileName,saveas,printSpikeTrainGraph):
        [NeuronsFired,SpikeTimes]=getSpikes(fileName) #convert the data in the PklFile to spikes

        ##write the spikes to a .sp file
        SpikeFileHandle = open("results/"+saveas+".sp",'w') 

        
        ##print out the spikes
        print "Neuron->SpikeTimes"
        SpikeFileHandle.write("Neuron->SpikeTimes\n")
        NumSpikes=len(NeuronsFired)
        for spikeNum in range (0,NumSpikes):
            SpikeFileHandle.write(str(NeuronsFired[spikeNum])+"->"+str(SpikeTimes[spikeNum])+"\n")
            print NeuronsFired[spikeNum], SpikeTimes[spikeNum]
        ##print spiketrains
        SpikeFileHandle.close()
        if(printSpikeTrainGraph==1):
            SpikeTrainGraph(fileName,saveas)            

        return [NeuronsFired,SpikeTimes]

def getSpikes(spikePklFile):
    #convert the data in the PklFile to spikes
    fileHandle = open(spikePklFile)
    neoObj = pickle.load(fileHandle)
    segments = neoObj.segments
    segment = segments[0]
    spikeTrains = segment.spiketrains
    neurons = len(spikeTrains)
    NeuronsFired=[]
    SpikeTimes=[]

    for neuronNum in range (0,neurons):
        if (len(spikeTrains[neuronNum])>0):
            spikes = spikeTrains[neuronNum]
            for spike in range (0,len(spikes)):
                NeuronsFired+=[neuronNum]
                SpikeTimes+=[float(spikes[spike])]
    fileHandle.close()    

    return[NeuronsFired,SpikeTimes]

def printSentenceFromSpikes(wordFile):
        [NeuronsFired,SpikeTimes]=getSpikes("results/genWords.pkl")
        ###read in word file
        Words=RetrieveWords(wordFile)
        WordNeuronsSorted=SortNeurons(NeuronsFired,SpikeTimes)  #Sort neurons in the order they spiked
        printSentence(Words,WordNeuronsSorted)


#--main
#def main():
args = sys.argv
numberArgs = args.__len__()
if (numberArgs == 4):
    
    fileName = args[1]
    spikeFileName=args[2]
    printSpikeTrain=int(args[3])
    [phraseNeuronsFired,phraseSpikeTimes]=printPklSpikes(fileName,spikeFileName,printSpikeTrain)
    printAll=0
    printSentenceFromSpikes("OutputWords.txt") #OutputWords.txt contains all of the different words comprising the language (sentences)
                                                #in a language file, e.g. chatbotLanguage.txt
                                                #the neural language generator creates the file OutputWords.txt 
else:
    
    printSpikeTrain = 1

    print "Phrase CA Spikes"
    [phraseNeuronsFired,phraseSpikeTimes]=printPklSpikes('results/genPhrase.pkl','SpikeTrainPhrases',printSpikeTrain)
    print "Sentence CA Spikes"
    [StateNeuronsFired,StateSpikeTimes]=printPklSpikes('results/genSentence.pkl','SpikeTrainSentenceStates',printSpikeTrain)
    print "Word Spikes"
    [WordNeuronsFired,WordSpikeTimes]=printPklSpikes('results/genWords.pkl','SpikeTrainWords',printSpikeTrain)
    printSentenceFromSpikes("OutputWords.txt")


