"""
This is the nest version of the world.  It's got a text box that you can
put inputs into.  These are then sent to the agent in the form of 
spikes, it processes them, and returns an action.  The communication is
done by text files:  words.txt for information from the environment to
the agent; actions.txt for information from the agent to the environment.
"""

import sys
import os.path
from Tkinter import Tk, Canvas, BOTH
from Tkinter import *

from agentWorldCommunicationNest import spikeQueue

import math
#import time

class Environment(Frame):
    direction = 0.0
    locationX = 0.0
    locationY = 0.0

    numFiles = 0
    numUserFiles = 0

    def __init__(self, parent):
        self.fileNum = 0
        Frame.__init__(self, parent)   
        self.toSpinQ = spikeQueue() #undone probably should rename
        
        #set up call back to interact with agent
        self.after(200,self.getNestInput)

        self.parent = parent        
        self.initUI()

    nestInputLines = 0
    #get the actions from the text file created in cabot3X.py
    def getNewNestInput(self,fileHandle):
        lineCount = 0
        for inpLine in fileHandle:
            lineCount += 1
            if (lineCount > self.nestInputLines):
                splitInp = inpLine.split()
                print "get action",inpLine,splitInp
                action = int(splitInp[0])
                fileHandle.close()
                if (splitInp.__len__() > 1):
                    actionTime = int(splitInp[1])
                    return [lineCount,action,actionTime]
                else:
                    return [lineCount,action,0]
                
        fileHandle.close()
        return [lineCount,-1]

    def getNestInput(self):
        if (os.path.isfile("actions.txt")):
            fileHandle = open("actions.txt",'r')
            newInput = self.getNewNestInput(fileHandle)
            if (self.nestInputLines == 0):
                self.toSpinQ.setTicks(0)
                self.nestInputLines = 1
                print 'turn step environment started'
            if (newInput[0] > self.nestInputLines):
                self.nestInputLines = newInput[0]
                action = newInput[1]
                #print 'new input',newInput
                print 'new input',self.nestInputLines,newInput[1]
                print 'act on', newInput[1], ' ', newInput[2]
                self.toSpinQ.setTicks(newInput[2])
                self.processCommandFromBoard(action)
        self.toSpinQ.increaseTimeTicks()
        self.after(200,self.getNestInput)

    def processTextInput (self, textInput):
        #clear whitespace before and after
        input = textInput.rstrip()
        input = input.lstrip()

        #test for characters.
        if input == "l":
            self.toSpinQ.writeSentence([-1,0,1,2],4)
        elif input == "f":
            self.toSpinQ.writeSentence([-1,3,4,2],4)
        else:
            print "bad text input ", textInput


    fPrintedCommand = False
    fPrintedNoCommand = False

    def processCommandFromBoard(self,commandNum):
        if (commandNum == 0):
            self.turnLeft()
        elif (commandNum == 1):
            self.turnRight()
        elif (commandNum == 2):
            self.moveForward()
        elif (commandNum == 3):
            self.moveBack()
        else:
            print ("environment ignores action ", commandNum)


    #-----Functions for moving
    def printPosition(self):
        print (self.direction,self.locationX,self.locationY)

    def moveForward(self):
        if self.direction == 0.0:
            self.locationX = self.locationX + 1.0
        elif self.direction == 90.0:
            self.locationY = self.locationY + 1.0
        elif self.direction == 180.0:
            self.locationX = self.locationX - 1.0
        elif self.direction == 270.0:
            self.locationY = self.locationY - 1.0
        self.printPosition()

    def moveBack(self):
        if self.direction == 0.0:
            self.locationX = self.locationX - 1.0
        elif self.direction == 90.0:
            self.locationY = self.locationY - 1.0
        elif self.direction == 180.0:
            self.locationX = self.locationX + 1.0
        elif self.direction == 270.0:
            self.locationY = self.locationY + 1.0
        self.printPosition()

    def turnLeft (self):
        self.direction = self.direction - 90.0
        self.direction = (self.direction % 360.0)
        self.printPosition()

    #send the sentence when you hit return
    def keyHandler(self,event):
        #print "keyhandler", repr(event.char)
        if (event.char == '\r'):
            text = self.editBox.get(1.0,END)
            self.editBox.delete(1.0,END)
            self.processTextInput(text)
            #self.putTextInFile(text)

    #set up the tickle frame with and edit box, buttons
    def initUI(self):
        self.parent.title("Turn Step Environment")        
        self.pack(fill=BOTH, expand=1)

        self.canvas = Canvas(self)

        self.canvas.pack(fill=BOTH, expand=1)

        quitButton = Button(self, text="Quit", command=self.quit)
        quitButton.place(x=0, y=0)
        printButton = Button(self, text="Print", command=self.printPosition)
        printButton.place(x=420, y=0)

        self.editBox = Text(self,height=1, width=50)
        self.editBox.place(x=0, y=25)
        self.editBox.bind("<Key>",self.keyHandler)
    #end of class

def main():
    print "main"
    root = Tk()
    env = Environment(root)
    root.geometry("500x200+300+300")
    root.mainloop()  


if __name__ == '__main__':
    main()  
